from ampersand import build, ampersand
import sys, os, json, pystache
args = sys.argv
p = os.path # Aliasing os.path to 'p'

def call_for_help(msg):

    if msg != "":
        print(msg)

    # Command usage
    print("\n** Ampersand - the minimal translation manager **\n")
    print("Usage: amp <command> [args]\n")
    print("   new <name> [lang] - Creates an empty Ampersand website")
    print("      compile <file> - Compiles the specified modal")
    print("    plugin <command> -  Manages plugins")
    print("            add <name> - Adds a plugin via Git")
    print("         remove <name> - Removes a plugin")
    print("               serve - Compiles all modals")

def amp():

    if len(args) > 1:
        if args[1] == "compile":

            print("Initializing the website...")
            site = ampersand.Ampersand()
            if len(args) > 2:
                # Compiles the specified web page
                site.compile(args[2])
            else:
                call_for_help("The command \"amp compile\" takes at least "
                            + "three arguments.")

        elif args[1] == "serve":
            # Serves all web pages
            print("Initializing the website...")
            site = ampersand.Ampersand()
            site.serve()

        elif args[1] == "new":
            if len(args) > 2:
                # Builds a new Ampersand project
                print("Creating new site '%s'" % (args[2]))
                lang = "en"
                if len(args) > 3:
                    lang = args[3]

                # Build the project tree
                print(" * Building tree")
                tree = [
                    "_modals", "_trans", p.join("_trans", lang),
                    "_layouts", "_site", "_plugins"
                ]
                try:
                    os.mkdir(args[2])
                except FileExistsError as e:
                    print(str(e))
                    sys.exit()

                for folder in tree:
                    os.mkdir(os.path.join(args[2], folder))

                # Create empty files
                open(p.join(args[2], "_modals/index.html"), "a+").close()
                f = open(p.join(args[2], "_trans", lang, "index.json"), "a+")
                f.write("{\n\n}")
                f.close()

                # Build the _ampersand.json file
                print(" * Building _ampersand.json")
                abspath = p.dirname(p.abspath(__file__))
                build.build_file(
                    p.join(abspath, "templates/_ampersand.json"),
                    args[2] + "/_ampersand.json", {
                        "name": args[2],
                        "lang": lang
                    }
                )

                print("Created boilerplate website.")

            else:
                call_for_help("The command \"amp new\" takes at least two "
                            + "arguments.")

        elif args[1] == "plugin":
            if len(args) > 3:
                site = ampersand.Ampersand()

                if args[2] == "add":
                    # Add a new plugin
                    site.plugin_add(args[3])

                elif args[2] == "remove":
                    # Remove an existing plugin
                    site.plugin_remove(args[3])

            else:
                call_for_help("The command \"amp plugin\" requires at least "
                            + "three arguments")

        else:
            call_for_help("That doesn't seem to be a valid command...")

    else:
        call_for_help("Missing arguments!")



from ampersand import build
from shutil import rmtree
import sys, os, json, subprocess, importlib, inspect
p = os.path

class Ampersand(object):

    """docstring for Ampersand."""

    def __init__(self):

        # Attempt to find the _ampersand.json configuration file
        try:
            config = build.get_json("_ampersand.json")
            root = p.dirname(p.abspath("./_ampersand.json"))

        except OSError:
            # Ask the user where to find the _ampersand.json file

            try:
                location = input("Enter the path (from here) to the root of "
                                + "your project: ")
                config = build.get_json(p.join(location, "_ampersand.json"))
                root = p.abspath(location)

            except (KeyboardInterrupt, OSError) as e:

                print(str(e))
                sys.exit()

        self.root = root
        self.config = config

    def serve(self):

        print(" * Collecting all pages")

        # Create an empty dictionary to store each pages content
        pages = {}
        for key, value in sorted(self.config["files"].items()):
            # Iterate through the files to build the pages dictionary
            pages[key] = build.collect(key, self)

        # Build the pages
        print(" * Building pages")
        build.build_pages(pages, self)

        print("Done.")

    def compile(self, filename):

        # Try to build the specified page
        print("Compiling page '%s'" % filename)
        try:
            print(" * Building page")
            build.build_pages({filename: build.collect(filename, self)}, self)

        except KeyError:
            # Exit if not found in _ampersand.json
            print("Didn't recognize %s as a file in _ampersand.json" % filename)
            sys.exit()

        print("Done.")

    def plugin_add(self, url):

        try:
            # Decide on what to call the plugin and its path
            plugin = p.splitext(p.split(url)[1])[0]
            plugin_path = p.join(self.root, self.config["modules"], plugin)

            # Download the plugin via git
            print("Installing Ampersand plugin '%s'" % plugin)
            try:
                clone = subprocess.check_call(["git", "clone", url, plugin_path])

                # Update the _ampersand.json file by adding the plugin
                self.config["plugins"][p.basename(plugin)] = p.join(
                    self.config["modules"], plugin )
                updated = open(p.join(self.root, "_ampersand.json"), "w")
                updated.write(json.dumps(self.config, indent=4))
                updated.close()
            except (subprocess.CalledProcessError, KeyboardInterrupt) as e:
                print(str(e))
                sys.exit()

        except KeyError as e:
            print("Missing entry in your configuration file: %s" % str(e))

    def plugin_remove(self, name):

        try:
            # Delete the directory containing the plugin
            print("Removing plugin '%s'" % name)
            rmtree(p.join( self.root, self.config["modules"], name ))
        except FileNotFoundError:
            pass

        try:
            # Update _ampersand.json by adding the plugin
            self.config["plugins"].pop(name)
            updated = open(p.join(self.root, "_ampersand.json"), "w")
            updated.write(json.dumps(self.config, indent=4))
            updated.close()
        except KeyError:
            print("Failed to remove plugin '%s' as it is not installed." % name)
            sys.exit()

    def plugin_run(self, name, content):

        try:
            # Retrieve the _plugin.json file
            plugin = build.get_json(
                p.join(self.root, self.config["plugins"][name], "_plugin.json"))

            # Load and run the module
            sys.path.append(p.join(self.root, self.config["plugins"][name]))
            module = importlib.import_module(plugin["init"], name)
            content = module.main(content, self)
            return content

        except (KeyError, FileNotFoundError,
                ImportError, AttributeError) as e:
            print("Failed to run plugin '%s': %s" % (name, e))
            return content



import sys, os, json, pystache
p = os.path # Aliasing os.path to 'p'

def read_file(path):
    # Open a file and return its contents
    f = open(path, "r")
    content = f.read()
    f.close()
    return content

def get_json(path):
    # Load a JSON file into a dictionary
    try:
        return json.loads(read_file(path))

    except json.decoder.JSONDecodeError as e:
        print("It seems like you have an error in your JSON file. Check that and try again.")
        print(str(e))
        sys.exit()

def build_file(modal, new_file, content):
    # Render the template HTML file
    new_content = pystache.render(read_file(modal), content)

    # Generate the new file using the template
    try:
        generated = open(new_file, "w")
    except FileNotFoundError:
        os.makedirs(p.dirname(new_file))
        generated = open(new_file, "w")
    generated.write(new_content)
    generated.close()

def collect(file_name, site):
    # Create variables pointing to items in the configuration
    root = site.root
    config = site.config

    template = config["files"][file_name]
    template_path = p.join(root, "_modals", file_name)
    translation = config["files"][file_name]
    build_dir = p.join(root, config["site"])

    pages = {}

    for key, value in sorted(template.items()):
        # Read the selected file's translation into the trans variable
        try:
            trans = get_json(p.join(root, config["files"][file_name][key]))

        except OSError as e:
            print(str(e))
            sys.exit()

        # Read _global.json into _globale
        try:
            _global = get_json(p.join(root, config["translations"], key, "_global.json"))

        except OSError:
            _global = {}


        layout_files = os.listdir(p.join(root, config["layouts"]))
        layouts = {}
        for i in range(len(layout_files)):
            # Read the layout into "contents"
            contents = read_file(p.join(root, config["layouts"], layout_files[i]))
            # Render the layouts using _ampersand.json and _global.json
            layouts[p.splitext(layout_files[i])[0]] = pystache.render(
                contents, {"config": config, "global": _global})

        # Assign the collected contents to the pages dictionary
        content = {"trans": trans, "layouts": layouts, "config": config, "global": _global}

        pages[key] = content

    return pages

def build_pages(content, site):
    config = site.config
    root = site.root

    # Iterate through the plugins
    for key, value in sorted(config["plugins"].items()):
        site.plugin_run(key, content)

    # Iterate through the files
    for key, value in sorted(content.items()):
        # Iterate through the translations
        for k, v in sorted(content[key].items()):
            # Set up the tree
            if k != config["primary"]:
                if not p.exists(p.join(root, config["site"], key)):
                    os.mkdir(p.join(root, config["site"], k))

            if k != config["primary"]: build_path = p.join(k, key)
            else: build_path = key

            # Build the file
            build_file(
                p.join(root, config["modals"], key),
                p.join(site.root, site.config["site"], build_path),
                content[key][k])
